import re


def clean_string(s: str) -> str:
    """
    Clean a string by removing extra whitespace characters.

    Args:
        s (str): The string to clean.

    Returns:
        str: The cleaned string.
    """
    # Replace all whitespace characters (newlines, tabs, multiple spaces) with a single space
    cleaned_string = re.sub(r"\s+", " ", s)
    # Remove leading and trailing spaces
    cleaned_string = cleaned_string.strip()
    return cleaned_string



import os
from openai import OpenAI
from datetime import datetime

from cvscout.models import Candidato, CANDIDATE_PARSER, CANDIDATE_FORMAT_INSTRUCTIONS


class LLM:
    def __init__(self, model: str = "gpt-4o"):
        """
        Initialize the LLM.

        Args:
            model (str): The model to use. Defaults to "gpt-4o".
        """
        assert (
            "OPENAI_API_KEY" in os.environ
        ), "Please set the OPENAI_API_KEY environment variable"

        self.llm = OpenAI(
            # This is the default and can be omitted
            api_key=os.environ.get("OPENAI_API_KEY"),
        )
        self.model = model
        self.system_template = """
        You are a human resources specialist.
        You must use spanish language.
        You have to perform the task of extracting information from CVs.
        You will be presented with a job title and the candidate's CV.

        Take into account that current date is {current_date}.

        Extract the data following the format:
        {format_instructions}
        """
        self.prompt_template = """
        Job title: {job_title}

        CV:
        {cv}
        """

    def __call__(self, job_title: str, cv: str) -> Candidato:
        """
        Analyze a CV and return a structured representation of the work experience.

        Args:
            job_title (str): The job title to analyze.
            cv (str): The CV to analyze.

        Returns:
            Candidato: The structured representation of the work experience.
        """
        # Get the current date
        current_date = datetime.now()
        # Format the date
        formatted_date = current_date.strftime("%d %B %Y")

        system_instructions = self.system_template.format(
            current_date=formatted_date,
            format_instructions=CANDIDATE_FORMAT_INSTRUCTIONS,
        )

        prompt = self.prompt_template.format(job_title=job_title, cv=cv)

        chat_completion = self.llm.chat.completions.create(
            messages=[
                {
                    "role": "system",
                    "content": system_instructions,
                },
                {
                    "role": "user",
                    "content": prompt,
                },
            ],
            model=self.model,
        )

        try:  # extract the response
            response = chat_completion.choices[0].message.content
            info_candidato: Candidato = CANDIDATE_PARSER.parse(response)
            return info_candidato
        except Exception as e:
            raise Exception(f"Error parsing response: {e}")



import math
from datetime import datetime
from dateutil.relativedelta import relativedelta
from langchain.output_parsers import PydanticOutputParser
from pydantic import BaseModel, Field, computed_field, field_validator

from cvscout.utils import clean_string


class ExperienciaLaboral(BaseModel):
    """Work experience data model."""

    empresa: str = Field(
        ...,
        description="Company name where the candidate worked.",
    )
    puesto: str = Field(
        ...,
        description="Job title of the candidate.",
    )
    fecha_inicio: datetime = Field(
        ...,
        description="Start date of the job experience.",
    )
    fecha_fin: datetime = Field(
        ...,
        description="End date of the job experience.",
    )
    relacionado: bool = Field(
        ...,
        description="""
        Whether the job experience is related to the job title.
        A job experience is considered related if the tasks to be performed
        are similar or related to the job title.
        """,
    )

    @computed_field
    @property
    def duracion(self) -> str:
        """Calculate the duration of the job experience."""
        delta = relativedelta(self.fecha_fin, self.fecha_inicio)
        # Format the difference
        parts = []
        if delta.years:
            parts.append(f"{delta.years} año{'s' if delta.years > 1 else ''}")
        if delta.months:
            parts.append(f"{delta.months} mes{'es' if delta.months > 1 else ''}")
        if delta.days:
            parts.append(f"{delta.days} día{'s' if delta.days > 1 else ''}")

        # Join the parts into a single string
        difference_str = " y ".join(parts)
        return difference_str

    @property
    def duracion_meses(self) -> int:
        """Calculate the duration of the job experience in months."""
        delta = relativedelta(self.fecha_fin, self.fecha_inicio)
        return delta.years * 12 + delta.months


class Candidato(BaseModel):
    """Candidate detail data model."""

    nombre: str = Field(
        ...,
        description="Candidate's name.",
    )
    edad: int = Field(
        ...,
        description="Candidate's age.",
    )
    experiencia: list[ExperienciaLaboral] = Field(
        ...,
        description="List of job experiences of the candidate.",
    )
    resumen: str = Field(
        ...,
        description="""
            Description of experience, an explanatory text about the candidate's experience.
            Please just take into account the job experiences related to the job title.
        """,
    )

    @field_validator("experiencia", mode="after")
    @classmethod
    def just_experiencia_related(
        cls, experiences: list[ExperienciaLaboral]
    ) -> list[ExperienciaLaboral]:
        """Remove job experiences that are not related to the job title."""
        return [exp for exp in experiences if exp.relacionado]

    @field_validator("resumen", mode="after")
    @classmethod
    def explainability_score(cls, resumen: str) -> str:
        """Generate an explanatory text about the candidate's experience."""
        explicacion_score = """
        El score asignado al candidato se calcula teniendo en cuenta la experiencia laboral relacionada
        con el puesto ofertado, premiando la fidelidad del candidato en las empresas en las que ha trabajado.
        """
        return clean_string(resumen + explicacion_score)

    @computed_field
    @property
    def score(self) -> int:
        """Calculate the score of the candidate."""
        # Take into account only the job experiences related to the job title
        related_experiences = [exp for exp in self.experiencia if exp.relacionado]

        total_relevant_months = 0

        # We will reward the loyalty of the candidate to the company
        # by applying a multiplier to the duration of the job experience
        for exp in related_experiences:
            months = exp.duracion_meses
            if months < 3:
                multiplier = 0.5
            elif months < 6:
                multiplier = 0.75
            elif months < 24:
                multiplier = 1.0
            else:
                multiplier = 1.25

            total_relevant_months += months * multiplier

        years_experinece = total_relevant_months / 12.0

        if years_experinece <= 0:
            score = 0
        elif years_experinece > 5:
            score = 100
        else:
            # score = 100 * math.sin(math.pi * (years_experinece - 0) / (10 - 0))
            score = 100 * (1 - math.exp(-1.25 * years_experinece))
        return int(score)


CANDIDATE_PARSER = PydanticOutputParser(pydantic_object=Candidato)
CANDIDATE_FORMAT_INSTRUCTIONS = CANDIDATE_PARSER.get_format_instructions()



from setuptools import setup, find_packages

with open("DESCRIPTION.md", "r") as f:
    description = f.read()

setup(
    name="cvscout",
    version="1.0",
    packages=find_packages(),
    install_requires=[
        "openai==1.30.1",
        "langchain==0.2.0",
        "pydantic==2.7.1",
    ],
    long_description=description,
    long_description_content_type="text/markdown",
)



from .models import Candidato
from .llm import LLM


class CVScout:
    def __init__(self, model: str = "gpt-4o"):
        """
        Initialize the CV Scout.

        Args:
            model (str): The model to use. Defaults to "gpt-4o".
        """
        self.llm = LLM(model)

    def __call__(self, job_title: str, cv: str) -> Candidato:
        """
        Analyze a CV and return a structured representation of the work experience.

        Args:
            job_title (str): The job title to analyze.
            cv (str): The CV to analyze.

        Returns:
            Candidato: The structured representation of the work experience.
        """
        return self.llm(job_title, cv)
